import re
from urlparse import urlparse

from django.http import Http404
from django.shortcuts import redirect

from courseware.courses import get_course_with_access


class Middleware(object):
    """
    This middleware is to keep the course nav bar above the wiki while
    the student clicks around to other wiki pages.
    If it intercepts a request for /wiki/.. that has a referrer in the
    form /courses/course_id/... it will redirect the user to the page
    /courses/course_id/wiki/...
    
    It is also possible that someone followed a link leading to a course
    that they don't have access to. In this case, we redirect them to the
    same page on the regular wiki.
    """
    
    def process_request(self, request):        
        referer = request.META.get('HTTP_REFERER')
        
        try:
            parsed_referer = urlparse(referer)
            referer_path = parsed_referer.path
        except:
            referer_path =""
                
        path_match = re.match(r'^/wiki/(?P<wiki_path>.*|)$', request.path)
        if path_match:
            # We are going to the wiki. Check if we came from a course
            course_match = re.match(r'/courses/(?P<course_id>[^/]+/[^/]+/[^/]+)/.*', referer_path)
            if course_match:
                course_id = course_match.group('course_id')
                
                # See if we are able to view the course. If we are, redirect to it
                try:
                    course = get_course_with_access(request.user, course_id, 'load')
                    return redirect("/courses/" + course.id + "/wiki/" + path_match.group('wiki_path') )
                    
                except Http404:
                    # Even though we came from the course, we can't see it. So don't worry about it.
                    pass
        
        else:
            # It is also possible we are going to a course wiki view, but we 
            # don't have permission to see the course!
            course_match = re.match(r'/courses/(?P<course_id>[^/]+/[^/]+/[^/]+)/wiki/(?P<wiki_path>.*|)$', request.path)
            if course_match:
                course_id = course_match.group('course_id')
                # See if we are able to view the course. If we aren't, redirect to regular wiki
                try:
                    course = get_course_with_access(request.user, course_id, 'load')
                    # Good, we can see the course. Carry on
                    return None
                except Http404:
                    # We can't see the course, so redirect to the regular wiki
                    return redirect("/wiki/" + course_match.group('wiki_path'))
            
        return None

def context_processor(request):
    """
    This is a context processor which looks at the URL while we are
    in the wiki. If the url is in the form
    /courses/(course_id)/wiki/...
    then we add 'course' to the context. This allows the course nav
    bar to be shown.
    """
    
    match = re.match(r'^/courses/(?P<course_id>[^/]+/[^/]+/[^/]+)/wiki(?P<wiki_path>.*|)', request.path)
    if match:
        course_id = match.group('course_id')
        
        try:
            course = get_course_with_access(request.user, course_id, 'load')
            return {'course' : course}
        except Http404:
            # We couldn't access the course for whatever reason. It is too late to change
            # the URL here, so we just leave the course context. The middleware shouldn't
            # let this happen
            pass
            
    return {}
    